# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Action générique
#
#
###########################################################################
from twisted.python import log
from ead2.backend.lib.action import * # import des actions  et types
from ead2.backend.actions import tools
from ead2.backend.actions.amon import amon_tools
from ead2.backend.config.config import TEMP_DIR
from ead2.backend.config.filenames import squidexceptions

import traceback
used_templates = ['main', 'form', 'accordion', 'formselect']

from os.path import join, isfile
sitelists = squidexceptions #join(TEMP_DIR, "squid_sitelist")

def reload_squid():
    """
    reload le service squid
    """
    cmd = "/etc/init.d/squid reload > /dev/null"
    code, value = tools.command_statusoutput(cmd)
    if code:
        return False
    return True

def del_site(site, _type="nocache"):
    """ supprime une référence à un site dans le fichier
    """
    sites = get_sites(_type)
    if site in sites:
        sites.remove(site)
        f_ = file(sitelists[_type], 'w')
        for site in sites:
            f_.write('.%s' % site)
            f_.write('\n')
        f_.close()
        return True
    raise Exception, "Erreur: le site n'est pas référencé."

def add_site(site, _type="nocache"):
    """
        ajoute un site dans le fichier de sites à interdire
    """
    site = amon_tools.format_url(site)
    if not site:
        raise Exception, "Erreur: veuillez rentrer une valeur"
    if site not in get_sites(_type):
        f_ = file(sitelists[_type], 'a')
        f_.write('.%s\n' % site)
        f_.close()
        return True
    raise Exception, "Erreur: le site est déjà référencé."

def get_sites(_type="nocache"):
    """ renvoie la liste des sites à ne pas authentifier
    """
    if isfile(sitelists[_type]):
        return [site[1:] for site in file(sitelists[_type]).read().splitlines() if site]
    return []

def get_form(server_nb, name):
    """ renvoie la description du formulaire d'entrée des sites à ne pas authentifier
    """
    sitename = {'name':'site', 'libelle':'Domaine à ajouter (sans le http)'}
    cache = {'name':'nocache', 'libelle':'Ne pas cacher ce site', 'inline':'ok', 'value':'ok'}
    auth = {'name':'noauth', 'libelle':'Ne pas authentifier ce site', 'inline':'ok', 'value':'ok'}
    validate = dict(href=tools.make_form_link(server_nb, name, True,
                                                ['addsite'],
                                                "squid_msg"),
                    libelle="Valider",
                    title="Ajouter ce site à cette liste",
                    icone="/image/ok.gif"
                    )
    return dict(sitename=sitename,
                cache=cache,
                auth=auth,
                validate=validate)

class SquidAuth(Action):
    """ Gestion d'une liste de sites à ne pas authentifier *
    """
    user_description = Dict(default={}, doc="description de l'éxécutant", keys=['ip', 'name', 'role'])
    name = "squid_sitelist" #celui de perm_nomdumodule.ini
    libelle = "Sites de mise à jour"
    category = "Configuration générale"
    description = "éditer la liste de sites à ne pas authentifier"

    request = Dict(default={}, doc="arguments de la requete en cours cote frontend",
                        keys=['server', 'action', 'todel', '_type'])#server et action sont les clés systématiques

    form_result = Dict(default={}, doc='retour de formulaire', keys=['addsite'])

    def execute(self):
        params, self.server_nb = tools.get_request(self.request)

        if params.has_key('todel') and params.has_key('_type'): # le nom du site à supprimer et le type : nocache ou noauth
            try:
                datas = self.valid_del(params['todel'][0].strip(), params['_type'][0].strip())
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                                     confirm=True)
            except Exception, message:
                traceback.print_exc()
                datas = dict(message=message.message)
            return self.send_frag(datas, template='amon_squid_sitelist',
                                         templates=used_templates)

        # Dans le cas d'un formulaire:
        if self.form_result.has_key('addsite'):
            try:
                # formatage de données pour traitement
                resultat = tools.format_form_result(self.form_result['addsite'])
                # datas: dico utilisé pour la templatisation
                datas = self.valid_form(resultat)
                datas['toexec'] = tools.make_js_link(self.server_nb, self.name,
                                                                     confirm=True)
            except Exception, message:
                traceback.print_exc()
                #si il y a une erreur soulevée par nos soins, on renvoie le message
                datas = dict(message=message.message)
            # Important: on vide le formulaire
            self.form_result = {}
            return self.send_frag(datas, template='NEW',
                                templates=used_templates)

        result = {} # variable renvoyée utilisée dans le template associé
        result.update(self.get_sites())
        result.update(get_form(self.server_nb, self.name))
        return self.send_all(result, template='amon_squid_sitelist',
                            templates=used_templates)

    def get_sites(self):
        """
            renvoie la liste des sites avec les données pour la mise en forme des boutons supprimer
        """
        sites = []
        cached = get_sites('nocache')
        authed = get_sites('noauth')

        for site in set(cached).union(set(authed)):

            if site in set(cached):
                href1 = tools.make_js_link(self.server_nb, self.name,
                                                balise="squid_msg",
                                                todel=site,
                                                _type='nocache')
            else:
                href1 = None

            if site in set(authed):
                href2 = tools.make_js_link(self.server_nb, self.name,
                                                balise="squid_msg",
                                                todel=site,
                                                _type='noauth')
            else:
                href2 = None

            icone = "/image/supprimer.gif"
            title = ""
            libelle =""
            _class = "btn"
            if href1:
                btn1 = dict(href=href1,icone=icone, title=title, libelle=libelle, _class=_class)
            else:
                btn1 = ''
            if href2:
                btn2 = dict(href=href2,icone=icone, title=title, libelle=libelle, _class=_class)
            else:
                btn2 = ''
            sites.append((site, btn1, btn2))
        return dict(sites=sites)

    def valid_form(self, result):
        """
            valide l'ajout de site à la liste des sites à ne pas authentifier et/ou cacher
        """
        sitename = result['site']
        sitename = amon_tools.format_url(sitename)
        result.pop('site')
        if result.keys():
            for key in result.keys():
                add_site(sitename, key)
            if not reload_squid():
                raise Exception, "Les modifications ont été appliquées au fichier de configuration mais le service n'a pas pu redémarrer"
        else:
            raise Exception, "Pensez à cocher une case"

        return dict()#dict(message="Le site %s a été ajouté à la liste de sites à ne pas authentifier."%sitename)

    def valid_del(self, sitename, _type):
        """
            valide la suppression de site de la liste des sites à ne pas autoriser
        """
        del_site(sitename, _type)
        if not reload_squid():
            raise Exception, "Les modifications ont été appliquées au fichier de configuration mais le service n'a pas pu redémarrer"
        return dict()#dict(message="Le site %s a été enlevé de la liste des sites à ne pas authentifier."%sitename)

