#-*-coding:utf-8-*-
"""
    Module d'édition des acls
"""
from os.path import isdir
from ead2.backend.config.config import module
# selon le module, les fonctions n'ont pas le même nom (et pas le même fonctionnement)
if module == 'horus':
    from horus.backend import get_groups as list_groups
    from horus.backend import get_users
    gr_types = ['']
else:
    from scribe.backend import list_groups
    from scribe.backend import get_users
    gr_types = ['Groupe', 'Matiere', "Responsables" , "Equipe", "Classe"]

from ead2.backend.actions.lib.acls.filesystem import deformat_path_from_html, format_path_for_html
from ead2.backend.actions import tools
# instruction javascript pour la suppression d'un élément dans la liste des acls à éditer
js_suppr = "javascript:setVoid('tr_%s_%s');"
js_search = "javascript:searchAndSelect('%s', '%s', '%s')"

# dictionnaire de translation du type à son suffixe html
keydict = {'r':'read', 'w':'write', 'x':'exec'}
inverted_keydict = dict([(value, key)for key, value in keydict.items()])

# commande pour la gestion des acls
mod_acl_cmd = "setfacl -RPm %s '%s'"
del_acl_cmd = "setfacl -RPx %s:%s '%s'"

## pour la mise à jour des acls par defaut
default_acl_cmd1 = "getfacl --access --absolute-names '%s'"
default_acl_cmd2 = "setfacl -RPd -M- '%s'"

# active les prints
debug = False

def get_acl_editor(server_nb, name, path):
    """ renvoie la description du formulaire d'édition des acls pour le path
        @path: chemin à éditer (fichier ou répertoire) au format dir__subdir__subsubdir
    """
    path = deformat_path_from_html(path)
    users = get_user_form()
    groups = get_groups_form()
    user_search = {'name':'user_search_input', 'libelle':'Rechercher un utilisateur', 'id':'user_search_input'}
    user_search_valid = {'href':js_search % ('user_search_input', 'datas', 'user'),
                         'title':'Chercher', 'icone':'/image/ok.gif',
                         'libelle':'', "_class":"btn"}
    group_search = {'name':'group_search_input', 'libelle':'Rechercher un groupe', 'id':'group_search_input'}
    group_search_valid = {'href':js_search % ('group_search_input', 'datas', 'group'),
                         'title':'Chercher', 'icone':'/image/ok.gif',
                         'libelle':'', "_class":"btn"}
    acls = get_path_acls(server_nb, name, path)
    btn = get_transf_btn("datas", "acls_form")
    close_btn = get_close_btn('acls_workspace')
    valid_btn = get_valid_btn(server_nb, name, path, "acls_form", "acls_workspace_msg")
    # construction du titre
    if isdir(path):
        acl_message = "Acls du répertoire"
    else:
        acl_message = "Acls du fichier"
    return dict(titre="Édition des acls de %s" % path,
                users=users,
                user_search=user_search,
                user_search_valid=user_search_valid,
                groups=groups,
                group_search=group_search,
                group_search_valid=group_search_valid,
                btn=btn,
                acl_message=acl_message,
                acls=acls,
                close_btn=close_btn,
                valid_btn=valid_btn
                )

def get_user_form():
    """
        renvoie les données pour la mise en forme de la partie du formulaire
        pour la gestion des acls par user
    """
    users = _get_users()
    users.sort()
    users_opts = [{'name':user,
              'libelle':user}
              for user in users]
    users_select = {'options':[{'name':'', 'options':users_opts}],
                     'libelle':'Utilisateurs',
                     'name':'users',
                     'id':'user',
                     'size':'20',
                     'multi':'ok'
                     }
    return users_select

def get_groups_form():
    """
        renvoie les données de mise en forme de la partie du formulaire
        pour la gestion des acls par groupe
    """

    groups = []
    for key in gr_types:
        group_names = _get_groups(key)
        group_names.sort()
        groups.append({'name':key, 'options':[{'name':gr, 'libelle':gr}for gr in group_names]})
    # le groupe professeurs ne fait partie d'aucun type de groupe
    if module == "scribe":
        groups.append({'name':"Autre", 'options':[
                            {'name':"eleves", "libelle":"eleves"},
                            {'name':"professeurs", "libelle":"professeurs"}
                                                 ]
                      })
    group_select = {'name':'group',
                    'libelle':'Choix du groupe',
                    'options':groups,
                    'id':'group',
                    'size':'20',
                    'multi':'ok'
                    }
    return group_select

def get_transf_btn(orig, dest):
    """ renvoie les données pour le transfert de données
        du formulaire orig vers le formulaire dest
        @orig: id du formulaire d'origine
        @dest: id du formulaire
    """
    href = "javascript:AclsTransferElement('%s', '%s')" % (orig, dest)
    title = "Ajouter des Acls"
    icone = "/image/leftright.png"
    libelle = ""
    _class = ""
    return dict(href=href,
                title=title,
                icone=icone,
                libelle=libelle,
                _class=_class)

def get_close_btn(divname):
    """
        renvoie la description du bouton pour la fermeture de la balise
    """
    href = "javascript:$('%s').style.display='none';setVoid('%s');"%(divname, divname)
    title = "Fermer"
    icone = "/image/supprimer.gif"
    libelle = "Fermer"
    _class = "close_btn"
    return dict(href=href,
                title=title,
                icone=icone,
                libelle=libelle,
                _class=_class)

def get_valid_btn(server_nb, name, path, formname, balise):
    """
        renvoie les données pour la mise en forme du bouton
        valider pour la modification des acls
    """
    href = tools.make_form_link(server_nb, name, True, [formname], balise, path=format_path_for_html(path))
    title = "Modifier les acls"
    libelle = "Valider"
    icone = "/image/ok.gif"
    return dict(href=href,
                title=title,
                libelle=libelle,
                icone=icone)

def get_path_acls(server_nb, name, path):
    """
        renvoie la description du formulaire d'édition
        des acls du chemin path
        renvoie {group_chekboxes:[(groupname, checkboxes)),
                 user_chekboxes:[(username, checkboxes)),
                 other:cheboxes,
                }
    """
    acls = parse_acls(path)
    groups = acls['groups']
    users = acls['users']
    other = acls['other']
    group_checkboxes = []
    user_checkboxes = []

    for group, rights in groups.items():
        group_checkboxes.append(get_checkboxes_and_suppr_btn(server_nb, name, group, rights, path, 'group'))
    for user, rights in users.items():
        user_checkboxes.append(get_checkboxes_and_suppr_btn(server_nb, name, user, rights, path, 'user'))
    other_checkboxes = get_checkboxes_and_suppr_btn(server_nb, name, 'Autre', other, path, 'other')

    return dict(group_checkboxes=group_checkboxes,
                user_checkboxes=user_checkboxes,
                other_checkboxes=other_checkboxes
                )

def get_checkboxes_and_suppr_btn(server_nb, name, item, rights, path, _type='group'):
    """
        renvoie les données pour la mise en forme de checkbox
        d'édition d'acls pour l'élément item
    """
    test = lambda key, rights: key in rights

    checkboxes = []
    for key in ('r', 'w', 'x'):
        suffix = keydict[key]
        check_name = "%s_%s_%s" % (item,_type, suffix)
        checkbox = {'name':check_name, 'libelle':'', 'value':check_name, 'inline':'ok'}
        if test(key, rights):
            checkbox['checked'] = 'ok'
        checkboxes.append(checkbox)

    suppr_btn = dict(href=tools.make_js_link(server_nb, name,
                                            balise='acls_workspace_msg',
                                             _type=_type,
                                             todel=item,
                                             path=format_path_for_html(path)
                                            ),
                    title="Supprimer les Acls pour %s" % item,
                    libelle="",
                    icone="/image/supprimer.gif",
                    _class="close_btn",
                    )

    return item, checkboxes, suppr_btn


def parse_acls(path):
    """
        parse le résultat de getfacl
    """
    status, acls_lines = tools.command_statusoutput("getfacl %s"%path)
    rights = {'groups':{}, 'users':{}}
    if debug:
        print "### Les acls de %s" % path

    for line in acls_lines.splitlines():
        line = line.strip()

        if line.startswith('# owner: '):
            owner = line[9:]
            if debug:
                print "+++ Le proprio est : ", owner

        elif line.startswith('# group: '):
            group_owner = line[9:]
            if debug:
                print "+++ Le groupe proprio : ", group_owner

        elif line.startswith('user::'):
            owner_rights = line[6:]
            if debug:
                print "+++ Les droits utilisateurs : ", owner_rights

        elif line.startswith('group::'):
            group_rights = line[7:]
            if debug:
                print "+++ Les droits du groupe : ", group_rights

        elif line.startswith('group:'):
            line = line[6:].split(':')
            if len(line) == 2:
                group = line[0]
                right = line[1].strip()
                rights['groups'][group] = right
                if debug:
                    print "++ groupe avec droit %s : %s" %(group, right)
            elif len(line) >2 : # cas où on a un #effective à la suite des droits du groupe (mask qui n'a rien à faire là)
                group = line[0]
                right = line[1].strip()
                if len(group)>0:
                    rights['groups'][group] = right
                    if debug:
                        print "++ groupe avec droit %s : %s" %(group, right)
            else:
                raise Exception, "Erreur au parsing d'Acls :récupération de nom de groupe"

        elif line.startswith('user:'):
            line = line[5:].split(':')
            if len(line) == 2:
                user = line[0]
                right = line[1]
                rights['users'][user] = right
                if debug:
                    print "++ utilisateurs avec droits %s : %s" % (user, right)
            elif len(line) >2 : # cas où on a un #effective à la suite des droits du groupe (mask qui n'a rien à faire là)
                user = line[0]
                right = line[1].strip()
                if len(user)>0:
                    rights['users'][user] = right
                    if debug:
                        print "++ utilisateurs avec droits %s : %s" % (user, right)
            else:
                raise Exception, "Erreur au parsing d'Acls : récupération de nom de user"

        elif line.startswith('other::'):
            right = line[7:]
            rights['other'] = right
            if debug:
                print "++ droits autres : %s" % right

    rights['users'][owner] = owner_rights
    rights['groups'][group_owner] = group_rights

    return rights

### VALIDATION DES MODIFICATIONS D'ACLS
def valid_acl_mod(result, path):
    """
        valide la modification d'acls pour le path : path
    """
    def make_acl_dict(result):
        """
            renvoie un dico {'u':{'username':'rwx'},
                             'g':{'groupname':'rwx'}}
        """
        type_keys = {'group':'g', 'user':'u'}
        get_right_type = lambda key: key.split('_')[-1]
        get_elem_type = lambda key:key.split('_')[-2]
        get_name = lambda key:'_'.join(key.split('_')[0:-2])
        translate_right = lambda key:inverted_keydict[key]
        acl_dict = {'u':{},
                    'g':{}}
        for key, value in result.items():
            _type = get_elem_type(key)
            right = get_right_type(key)
            name = get_name(key)
            _type = type_keys[_type]
            acl_dict[_type].setdefault(name, {}).setdefault(translate_right(right),value)
        return acl_dict

    def valid_right(right, value):
        """
            renvoie les droits
        """
        if value: return right
        else: return '-'
    # on remplace les __ par des slashs
    path = deformat_path_from_html(path)
    acls_template = "%(_type)s:%(name)s:%(r)s%(w)s%(x)s"
    acls_lines = []
    for _type, elements in make_acl_dict(result).items():
        for name, rights in elements.items():
            acls_lines.append(acls_template % dict(_type=_type,
                                                   name=name,
                                                   r=valid_right('r',rights['r']),
                                                   w=valid_right('w',rights['w']),
                                                   x=valid_right('x',rights['x'])))
    for cmd in acls_lines:
        setacl(cmd, path)
    return dict()

def valid_del_acl(item, _type, path):
    """
        supprime les acls pour l'item de type _type pour le path path
    """
    path = deformat_path_from_html(path)
    type_keys = {'group':'g', 'user':'u'}

    cmd = del_acl_cmd % (type_keys[_type], item, path)

    a,b = tools.command_statusoutput(cmd)
    if a:
        raise Exception, "Erreur à l'éxécution de %s : %s " % (cmd, b)
    if isdir(path):
        set_default_acl(path)
    return dict()


def setacl(cmd, path):
    """
        éxécute la commande setacl
    """
    cmd = mod_acl_cmd % (cmd, path)

    a,b = tools.command_statusoutput(cmd)
    if a:
        raise Exception, "Erreur à la mise en place des acls \\n Éxécution de : %s\\n Message retour : \\n %s " % (cmd, b)
    if isdir(path):
        set_default_acl(path)
    return True


def set_default_acl(path):
    """
        met à jour les acls par défaut pour l'héritage
        pour les repertoires mais ne gueule pas pour les fichiers
    """
    cmd1 = default_acl_cmd1 % path
    cmd2 = default_acl_cmd2 % path
    a,b = tools.command_statusoutput(cmd1, cmd2)
    if a:
        raise Exception, "Erreur à la mise à jour des acls par défaut :\\n %s" % b
    return True

def _get_groups(key=''):
    """
        fonction de mapping pour gérer le bi-module (horus, scribe)
        renvoie la liste des groupes pour la clé key
    """
    if key: # cas scribe
        return list_groups(key)
    else: # cas horus
        return list_groups()

def _get_users():
    """
        fonction de mapping pour gérer le bi-module (horus, scribe)
        renvoie la liste des utilisateurs
    """
    return get_users()
