# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Gère de groupe de services
#
###########################################################################
""" gestionnaire de groupes de services """
from ead2.backend.config.filenames import simple_services as services_list
from sets import Set
from ConfigParser import ConfigParser
from ead2.backend.config.config import module
from creole import fonctionseole

def get_available_services():
    """ renvoie la liste des services
        listé dans le répertoire /etc/sysconfig/eole/services/*.srv
    """
    list_services = fonctionseole.get_services_actifs()
    list_services.sort()
    return list_services

class ServicesParser(ConfigParser):
    """ parser pour le fichier de configuration des groupes de services """
    def parse_file(self, filename):
        """ parse un fichier """
        self.read(filename)

    def get_services(self):
        """ renvoie les groupes de services """
        if module in self.sections():
            for libelle, services in self.items(module):
                yield({'libelle':libelle, 'services':services.split(',')})

    def get_libelles(self):
        """ renvoie la liste des groupes de services """
        if module in self.sections():
            for libelle, toto in self.items(module):
                yield libelle

def get_group_services(group_name):
    """ renvoie la liste des services associés à un groupe """
    sparser = ServicesParser()
    sparser.parse_file(services_list)
    for i in sparser.get_services():
        if i['libelle'] == group_name:
            list_services = i['services']
            list_services.sort()
            return list_services
    return []

def get_groups():
    """ renvoie la liste des groupes du module """
    sparser = ServicesParser()
    sparser.parse_file(services_list)
    return list(sparser.get_libelles())

def get_group_available_services(group_name):
    """ renvoie la différence entre la liste des services
        du groupe et des services disponibles
    """
    list_services = list(Set(get_available_services()).difference(Set(get_group_services(group_name))))
    list_services.sort()
    return list_services

## action sur le fichier de config
def create_service_group(group_name, services):
    """ cree un groupe de service """
    if type(services) != list:
        services = [services]
    if group_name in get_groups():
        raise Exception, "Ce groupe de service existe déjà sur %s." % module
    if not services :
        raise Exception, "Veuiller associer au moins un service à la création de ce groupe."
    sparser = ServicesParser()
    sparser.parse_file(services_list)
    if not sparser.has_section(module):
        sparser.add_section(module)
    sparser.set(module, group_name, ','.join(services))
    fic = file(services_list, 'w')
    sparser.write(fic)
    fic.close()

def mod_service_group(group_name, services):
    """ modifie un groupe de services """
    if type(services) != list:
        services = [services]
    if list(Set(get_group_services(group_name)).difference(Set(services))) == [] and list(Set(services).difference(Set(get_group_services(group_name)))) == []:
        # pas de modification
        return True
    sparser = ServicesParser()
    sparser.parse_file(services_list)
    if not sparser.has_section(module):
        sparser.add_section(module)
    sparser.remove_option(module, group_name)
    if services:
        sparser.set(module, group_name, ','.join(services))
    fic = file(services_list, 'w')
    sparser.write(fic)
    fic.close()

def del_service_group(group_name):
    """ supprime un groupe de services """
    if group_name not in get_groups():
        raise Exception, "Erreur : Ce groupe n'existe pas, impossible de le supprimer."
    sparser = ServicesParser()
    sparser.parse_file(services_list)
    sparser.remove_option(module, group_name)
    fic = file(services_list, 'w')
    sparser.write(fic)
    fic.close()
